@@ -9,7 +9,7 @@ if ( $] < 5.010000 ) {
 package LINQ::Array;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.000_004';
+our $VERSION   = '0.000_005';
 
 use Class::Tiny qw( array );
 use Role::Tiny::With ();
@@ -32,8 +32,4 @@ sub to_list {
 	@$self;
 }
 
-sub target_class {
-	__PACKAGE__;
-}
-
 1;
