@@ -103,7 +103,7 @@ if ( $] < 5.010000 ) {
 package LINQ::Iterator;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.000_004';
+our $VERSION   = '0.000_005';
 
 use Role::Tiny::With ();
 use LINQ::Util::Internal ();
@@ -116,7 +116,10 @@ sub new {
 		tie my ( @arr ), 'LINQ::Iterator::_LazyList', LINQ::Util::Internal::assert_code( @_ );
 		return bless \@arr, $class;
 	}
-	die "Expected to be given a CODE ref!";
+	LINQ::Util::Internal::throw(
+		"CallerError",
+		message => "Expected a coderef"
+	);
 }
 
 sub _guts {
@@ -124,11 +127,6 @@ sub _guts {
 	tied( @$self );
 }
 
-sub target_class {
-	require LINQ::Array;
-	"LINQ::Array";
-}
-
 sub to_list {
 	my $self = shift;
 	my @list = @$self;
@@ -164,7 +162,7 @@ sub select {
 	if ( my $guts = $self->_guts ) {
 		my $map = LINQ::Util::Internal::assert_code( @_ );
 		my $idx = 0;
-		return __PACKAGE__->new( sub {
+		return LINQ::Util::Internal::create_linq( sub {
 			my $val = $guts->fetch_ref( $idx++ );
 			if ( ! $val ) {
 				require LINQ;
@@ -184,7 +182,7 @@ sub where {
 	if ( my $guts = $self->_guts ) {
 		my $check = LINQ::Util::Internal::assert_code( @_ );
 		my $idx   = 0;
-		return __PACKAGE__->new( sub {
+		return LINQ::Util::Internal::create_linq( sub {
 			GETVAL: {
 				my $val = $guts->fetch_ref( $idx++ );
 				if ( ! $val ) {
