@@ -5,12 +5,11 @@ use warnings;
 package LINQ::Util::Internal;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.000_004';
+our $VERSION   = '0.000_005';
 
 sub create_linq {
-	my $parent = shift;
-	my $class  = $parent->target_class;
-	ref( $class ) ? $class->( @_ ) : $class->new( @_ );
+	require LINQ;
+	goto \&LINQ::LINQ;
 }
 
 sub throw {
